#include "emprestimo.h"
#include "pesquisa.h"
#include "time.h"

void registrarEmprestimo(Emprestimo **registos, int *numRegistos, Livro *biblioteca, int numLivros) {
    char utilizador[MAX_UTILIZADOR];
    int diasEmprestimo;

    // Limpa o buffer do teclado
    int c;
    while ((c = getchar()) != '\n' && c != EOF);

    // Solicita e lê o nome do sócio
    printf("Digite o nome do sócio: ");
    fgets(utilizador, sizeof(utilizador), stdin);
    utilizador[strcspn(utilizador, "\n")] = '\0'; // Remove o caractere de nova linha, se presente

    // Realiza a pesquisa de livro
    pesquisarLivro(biblioteca, numLivros, *registos, *numRegistos);

    int escolhaLivro;
    do {
        // Solicita e lê o número do livro a ser emprestado
        printf("\nEscolha o número do livro que deseja emprestar: ");
        scanf("%d", &escolhaLivro);
        if (escolhaLivro < 1 || escolhaLivro > numLivros) {
            printf("Escolha inválida. Por favor, tente novamente.\n");
        }
    } while (escolhaLivro < 1 || escolhaLivro > numLivros);

    escolhaLivro--; // Ajusta o índice do vetor para corresponder à escolha do usuário

    // Solicita e lê o número de dias do empréstimo
    printf("Digite o número de dias do empréstimo: ");
    scanf("%d", &diasEmprestimo);
    getchar(); // Limpa o buffer do teclado

    // Incrementa o número de registros e realoca memória conforme necessário
    (*numRegistos)++;
    *registos = (Emprestimo *)realloc(*registos, (*numRegistos) * sizeof(Emprestimo));
    if (*registos == NULL) {
        printf("Erro ao alocar memória!\n");
        return; // Sai da função em caso de falha na alocação de memória
    }

    // Cria um novo registro de empréstimo e preenche seus dados
    Emprestimo novoEmprestimo;
    strcpy(novoEmprestimo.utilizador, utilizador);
    strcpy(novoEmprestimo.tituloLivro, biblioteca[escolhaLivro].titulo);
    novoEmprestimo.diasEmprestimo = diasEmprestimo;

    // Obtém a data atual e calcula a data de devolução
    int dataAtual = getDataAtual();
    novoEmprestimo.dataDevolucao = dataAtual + diasEmprestimo;

    novoEmprestimo.devolvido = 0; // Define que o livro ainda não foi devolvido

    // Adiciona o novo registro de empréstimo ao vetor de registros
    (*registos)[*numRegistos - 1] = novoEmprestimo;

    // Decrementa o número de exemplares disponíveis do livro emprestado
    biblioteca[escolhaLivro].exemplares--;

    // Informa o utilizador que o empréstimo foi registrado com sucesso
    printf("Empréstimo registrado com sucesso!\n");
}

void removerEmprestimo(Emprestimo *registos, int *numRegistos) {
    char termoPesquisa[100];
    int n=1;

    // Mostra os empréstimos não devolvidos para o utilizador escolher qual remover
    printf("ID - Nome do Sócio, Livro, Dias restantes:\n");
    for (int i = 0; i < *numRegistos; i++) {
        if (!registos[i].devolvido) {
            printf("%d - %s, %s, %d\n", n, registos[i].utilizador, registos[i].tituloLivro, registos[i].diasEmprestimo);
            n++;
        }
    }

    // Solicita e valida a escolha do utilizador para o empréstimo a ser removido
    int escolhaEmprestimo;
    do {
        printf("\nEscolha o número do empréstimo que deseja remover: ");
        scanf("%d", &escolhaEmprestimo);
        if (escolhaEmprestimo < 1 || escolhaEmprestimo > *numRegistos) {
            printf("Escolha inválida. Por favor, tente novamente.\n");
        }
    } while (escolhaEmprestimo < 1 || escolhaEmprestimo > *numRegistos);

    // Encontra o índice do empréstimo escolhido para remover
    int indiceEmprestimo = -1;
    int ID = 1;
    for (int i = 0; i < *numRegistos; i++) {
        if (!registos[i].devolvido) {
            // Verifica se o empréstimo corresponde ao número escolhido pelo utilizador
            if (ID == escolhaEmprestimo) {
                indiceEmprestimo = i;
                break;
            }
            ID++;
        }
    }

    // Se o índice do empréstimo não for encontrado, exibe uma mensagem de erro e retorna
    if (indiceEmprestimo == -1) {
        printf("Índice de empréstimo inválido.\n");
        return;
    }

    // Marca o empréstimo como devolvido
    registos[indiceEmprestimo].devolvido = 1;

    // Informa ao utilizador que o empréstimo foi removido com sucesso
    printf("Empréstimo removido com sucesso!\n");
}

void listarEmprestimosAtivos(Emprestimo *registos, int numRegistos) {
    // Imprime um cabeçalho para a lista de empréstimos ativos
    printf("Lista de Empréstimos Ativos:\n");
    
    // Percorre todos os registros de empréstimos
    for (int i = 0; i < numRegistos; i++) {
        // Verifica se o empréstimo ainda não foi devolvido
        if (!registos[i].devolvido) {
            // Se o empréstimo estiver ativo, imprime informações sobre ele
            printf("Sócio: %s, Livro: %s, Dias: %d\n", registos[i].utilizador, registos[i].tituloLivro, registos[i].diasEmprestimo);
        }
    }
}

void lerEmprestimos(const char *nomeArquivo, Emprestimo **registos, int *numRegistos) {
    // Abre o arquivo no modo leitura
    FILE *arquivo = fopen(nomeArquivo, "r");
    if (arquivo == NULL) {
        // Verifica se houve erro na abertura do arquivo
        printf("Erro ao abrir o arquivo %s.\n", nomeArquivo);
        return; // Retorna da função em caso de erro
    }
    
    // Move o cursor para o fim do arquivo para determinar o tamanho do arquivo
    fseek(arquivo, 0, SEEK_END);
    long fileSize = ftell(arquivo);
    rewind(arquivo); // Retorna o cursor para o início do arquivo
    
    // Calcula o número de registros de empréstimos com base no tamanho do arquivo
    *numRegistos = fileSize / sizeof(Emprestimo);
    
    // Aloca memória para armazenar os registros de empréstimos
    *registos = (Emprestimo *)malloc((*numRegistos) * sizeof(Emprestimo));
    if (*registos == NULL) {
        // Verifica se houve erro na alocação de memória
        printf("Erro ao alocar memória!\n");
        fclose(arquivo); // Fecha o arquivo antes de retornar da função
        return; // Retorna da função em caso de erro na alocação de memória
    }
    
    // Lê os registros de empréstimos do arquivo e armazena-os na memória
    fread(*registos, sizeof(Emprestimo), *numRegistos, arquivo);
    
    // Fecha o arquivo após a leitura
    fclose(arquivo);
}

void gravarEmprestimos(const char *nomeArquivo, Emprestimo *registos, int numRegistos) {
    // Abre o arquivo no modo escrita, com a opção para especificar a codificação UTF-8
    FILE *arquivo = fopen(nomeArquivo, "w, css=UTF-8");
    if (arquivo == NULL) {
        // Verifica se houve erro na criação do arquivo
        printf("Erro ao criar o arquivo %s.\n", nomeArquivo);
        return; // Retorna da função em caso de erro
    }
    
    // Escreve os registros de empréstimos no arquivo
    fwrite(registos, sizeof(Emprestimo), numRegistos, arquivo);
    
    // Fecha o arquivo após a escrita
    fclose(arquivo);
}