#include "livro.h"

void adicionarLivro(Livro **biblioteca, int *numLivros) {
    // Incrementa o número total de livros
    (*numLivros)++;
    
    // Realoca memória para a biblioteca de livros
    *biblioteca = (Livro *)realloc(*biblioteca, (*numLivros) * sizeof(Livro));
    if (*biblioteca == NULL) {
        // Verifica se houve erro na alocação de memória
        printf("Erro ao alocar memória!\n");
        return; // Retorna da função em caso de erro
    }
    
    // Cria um novo livro
    Livro novoLivro;
    
    // Solicita e lê o título do livro
    printf("Título do livro: ");
    getchar(); // Limpa o buffer do teclado
    fgets(novoLivro.titulo, sizeof(novoLivro.titulo), stdin);
    novoLivro.titulo[strcspn(novoLivro.titulo, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    // Solicita e lê o autor do livro
    printf("Autor do livro: ");
    fgets(novoLivro.autor, sizeof(novoLivro.autor), stdin);
    novoLivro.autor[strcspn(novoLivro.autor, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    // Solicita e lê o género do livro
    printf("Género do livro: ");
    fgets(novoLivro.genero, sizeof(novoLivro.genero), stdin);
    novoLivro.genero[strcspn(novoLivro.genero, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    // Solicita e lê o número de exemplares do livro
    printf("Número de exemplares: ");
    scanf("%d", &novoLivro.exemplares);
    
    // Adiciona o novo livro à biblioteca
    (*biblioteca)[*numLivros - 1] = novoLivro;
    
    // Informa ao utilizador que o livro foi adicionado com sucesso
    printf("Livro adicionado com sucesso!\n");
}

void removerLivro(Livro **biblioteca, int *numLivros) {
    // Verifica se a biblioteca está vazia
    if (*numLivros == 0) {
        printf("A biblioteca está vazia. Não há livros para remover.\n");
        return; // Retorna da função se não houver livros na biblioteca
    }
    
    // Mostra a lista de livros disponíveis na biblioteca
    printf("Livros na biblioteca:\n");
    for (int i = 0; i < *numLivros; i++) {
        printf("%d. %s\n", i + 1, (*biblioteca)[i].titulo);
    }
    
    // Solicita e lê o número do livro a ser removido
    int numLivro;
    printf("Digite o número do livro a ser removido, ou 0 para cancelar: ");
    scanf("%d", &numLivro);

    // Verifica se o utilizador cancelou a operação
    if (numLivro == 0) {
        return; // Retorna da função se o utilizador cancelou a operação
    }
    
    // Verifica se o número do livro é válido
    if (numLivro < 1 || numLivro > *numLivros) {
        printf("Número de livro inválido!\n");
        return; // Retorna da função se o número do livro for inválido
    }
    
    // Remove o livro da posição numLivro - 1
    for (int i = numLivro - 1; i < *numLivros - 1; i++) {
        (*biblioteca)[i] = (*biblioteca)[i + 1];
    }
    (*numLivros)--; // Decrementa o número total de livros na biblioteca
    
    // Realoca memória para o vetor de livros
    *biblioteca = (Livro *)realloc(*biblioteca, (*numLivros) * sizeof(Livro));
    if (*numLivros > 0 && *biblioteca == NULL) {
        printf("Erro ao alocar memória!\n");
        return; // Retorna da função em caso de erro na alocação de memória
    }
    
    // Informa ao utilizador que o livro foi removido com sucesso
    printf("Livro removido com sucesso!\n");
}

void editarLivro(Livro *biblioteca, int numLivros) {
    // Verifica se a biblioteca está vazia
    if (numLivros == 0) {
        printf("A biblioteca está vazia. Não há livros para editar.\n");
        return; // Retorna da função se não houver livros na biblioteca
    }
    
    // Mostra a lista de livros disponíveis na biblioteca
    printf("Livros na biblioteca:\n");
    for (int i = 0; i < numLivros; i++) {
        printf("%d. %s\n", i + 1, biblioteca[i].titulo);
    }
    
    // Solicita e lê o número do livro a ser editado
    int numLivro;
    printf("Digite o número do livro a ser editado ou 0 para cancelar: ");
    scanf("%d", &numLivro);

    // Verifica se o utilizador cancelou a operação
    if (numLivro == 0) {
        return; // Retorna da função se o utilizador cancelou a operação
    }
    
    // Verifica se o número do livro é válido
    if (numLivro < 1 || numLivro > numLivros) {
        printf("Número de livro inválido!\n");
        return; // Retorna da função se o número do livro for inválido
    }
    
    // Solicita e lê os novos dados do livro a ser editado
    printf("Novo título do livro: ");
    getchar(); // Limpa o buffer do teclado
    fgets(biblioteca[numLivro - 1].titulo, sizeof(biblioteca[numLivro - 1].titulo), stdin);
    biblioteca[numLivro - 1].titulo[strcspn(biblioteca[numLivro - 1].titulo, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    printf("Novo autor do livro: ");
    fgets(biblioteca[numLivro - 1].autor, sizeof(biblioteca[numLivro - 1].autor), stdin);
    biblioteca[numLivro - 1].autor[strcspn(biblioteca[numLivro - 1].autor, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    printf("Novo género do livro: ");
    fgets(biblioteca[numLivro - 1].genero, sizeof(biblioteca[numLivro - 1].genero), stdin);
    biblioteca[numLivro - 1].genero[strcspn(biblioteca[numLivro - 1].genero, "\n")] = '\0'; // Remove o caractere de nova linha, se presente
    
    printf("Novo número de exemplares: ");
    scanf("%d", &biblioteca[numLivro - 1].exemplares);
    
    // Informa ao utilizador que o livro foi editado com sucesso
    printf("Livro editado com sucesso!\n");
}

void lerLivros(const char *nomeArquivo, Livro **biblioteca, int *numLivros) {
    // Abre o arquivo no modo leitura
    FILE *arquivo = fopen(nomeArquivo, "r");
    if (arquivo == NULL) {
        // Verifica se houve erro na abertura do arquivo
        printf("Erro ao abrir o arquivo %s.\n", nomeArquivo);
        return; // Retorna da função em caso de erro
    }
    
    // Move o cursor para o fim do arquivo para determinar o tamanho do arquivo
    fseek(arquivo, 0, SEEK_END);
    long fileSize = ftell(arquivo);
    rewind(arquivo); // Retorna o cursor para o início do arquivo
    
    // Calcula o número de livros com base no tamanho do arquivo
    *numLivros = fileSize / sizeof(Livro);
    
    // Aloca memória para armazenar os livros
    *biblioteca = (Livro *)malloc((*numLivros) * sizeof(Livro));
    if (*biblioteca == NULL) {
        // Verifica se houve erro na alocação de memória
        printf("Erro ao alocar memória!\n");
        fclose(arquivo); // Fecha o arquivo antes de retornar da função
        return; // Retorna da função em caso de erro na alocação de memória
    }
    
    // Lê os livros do arquivo e armazena-os na memória
    fread(*biblioteca, sizeof(Livro), *numLivros, arquivo);
    
    // Fecha o arquivo após a leitura
    fclose(arquivo);
}

void gravarLivros(const char *nomeArquivo, Livro *biblioteca, int numLivros) {
    // Abre o arquivo para escrita, com a opção para especificar a codificação UTF-8
    FILE *arquivo = fopen(nomeArquivo, "w, css=UTF-8");
    if (arquivo == NULL) {
        // Verifica se houve erro na criação do arquivo
        printf("Erro ao criar o arquivo %s.\n", nomeArquivo);
        return; // Retorna da função em caso de erro
    }
    
    // Escreve os livros no arquivo
    fwrite(biblioteca, sizeof(Livro), numLivros, arquivo);
    
    // Fecha o arquivo após a escrita
    fclose(arquivo);
}