#include "pesquisa.h"
#include "livro.h"
#include "emprestimo.h"

// Função para pesquisar livros na biblioteca
void pesquisarLivro(Livro *biblioteca, int numLivros, Emprestimo *registos, int numRegistos) {
    // Verifica se a biblioteca está vazia
    if (numLivros == 0) {
        printf("A biblioteca está vazia. Não há livros para pesquisar.\n");
        return;
    }
    
    char termoPesquisa[100];
    // Solicita ao utilizador o termo de pesquisa
    printf("Digite o título, autor ou género do livro a ser pesquisado: ");
    getchar();
    fgets(termoPesquisa, sizeof(termoPesquisa), stdin);
    termoPesquisa[strcspn(termoPesquisa, "\n")] = '\0'; // Remove o caractere de nova linha
    
    int encontrados = 0;

    // Percorre os livros na biblioteca
    for (int i = 0; i < numLivros; i++) {
        int exemplaresDisponiveis = biblioteca[i].exemplares;
        // Percorre os registos de empréstimos
        for (int j = 0; j < numRegistos; j++) {
            // Verifica se o livro está emprestado
            if (!registos[j].devolvido && strcmp(registos[j].tituloLivro, biblioteca[i].titulo) == 0) {
                exemplaresDisponiveis--;
            }
        }
        // Verifica se o livro corresponde ao termo de pesquisa e se há exemplares disponíveis
        if (strstr(biblioteca[i].titulo, termoPesquisa) != NULL ||
            strstr(biblioteca[i].autor, termoPesquisa) != NULL ||
            strstr(biblioteca[i].genero, termoPesquisa) != NULL) {
            if (exemplaresDisponiveis > 0) {
                // Imprime os detalhes do livro
                printf("%d. %s, %s, %s, %d exemplares\n", i + 1, biblioteca[i].titulo, biblioteca[i].autor,
                       biblioteca[i].genero, exemplaresDisponiveis);
                encontrados++;
            }
        }
    }

    // Informa se nenhum livro foi encontrado com o termo de pesquisa
    if (encontrados == 0) {
        printf("Nenhum livro encontrado com o termo '%s'.\n", termoPesquisa);
    }
}

// Função para pesquisar empréstimos na lista de registos
void pesquisarEmprestimo(Emprestimo *registos, int numRegistos) {
    char termoPesquisa[100];
    // Solicita ao utilizador o termo de pesquisa
    printf("Digite o título, autor ou género do livro a ser pesquisado: ");
    getchar();
    fgets(termoPesquisa, sizeof(termoPesquisa), stdin);
    termoPesquisa[strcspn(termoPesquisa, "\n")] = '\0'; // Remove o caractere de nova linha

    int encontrados = 0;

    // Percorre os registos de empréstimos
    for (int i = 0; i < numRegistos; i++) {
        // Verifica se o título do livro ou o nome do utilizador correspondem ao termo de pesquisa
        if (strstr(registos[i].tituloLivro, termoPesquisa) != NULL ||
            strstr(registos[i].utilizador, termoPesquisa) != NULL) {
            // Imprime os detalhes do empréstimo
            printf("%d. Livro: %s, Utilizador: %s, Dias: %d\n", i + 1, registos[i].tituloLivro, registos[i].utilizador,
                   registos[i].diasEmprestimo);
            encontrados++;
        }
    }

    // Informa se nenhum empréstimo foi encontrado com o termo de pesquisa
    if (encontrados == 0) {
        printf("Nenhum empréstimo encontrado com o termo '%s'.\n", termoPesquisa);
    }
}