#include <stdio.h>
#include <string.h>
#include "relatorio.h"
#include "time.h"

void relatorioLivrosMaisEmprestados(Emprestimo *registos, int numRegistos, Livro *biblioteca, int numLivros) {
    printf("Relatório dos 10 livros mais emprestados:\n");

    // Inicializa um array para contar o número de empréstimos de cada livro
    int contador[numLivros];
    memset(contador, 0, sizeof(contador)); // Inicializa com 0

    // Conta o número de empréstimos de cada livro
    for (int i = 0; i < numRegistos; i++) {
        for (int j = 0; j < numLivros; j++) {
            if (strcmp(registos[i].tituloLivro, biblioteca[j].titulo) == 0) {
                contador[j]++;
                break;
            }
        }
    }

    // Encontra e imprime os 10 livros mais emprestados
    for (int k = 0; k < 10; k++) {
        int maxEmpr = -1;
        int maxIndex = -1;
        for (int i = 0; i < numLivros; i++) {
            if (contador[i] > maxEmpr) {
                maxEmpr = contador[i];
                maxIndex = i;
            }
        }

        if (maxIndex != -1) {
            printf("%d. %s: %d empréstimos\n", k + 1, biblioteca[maxIndex].titulo, maxEmpr);
            contador[maxIndex] = -1; // Marcador para evitar que o mesmo livro seja contado novamente
        }
    }
}

void relatorioLivrosNaoDevolvidos(Emprestimo *registos, int numRegistos, Livro *biblioteca, int numLivros, int dataAtual) {
    printf("Relatório dos livros não devolvidos:\n");

    // Variável para rastrear se pelo menos um empréstimo foi encontrado
    int emprestimosEncontrados = 0;

    // Percorre os empréstimos para encontrar os não devolvidos e imprime suas informações
    for (int i = 0; i < numRegistos; i++) {
        if (!registos[i].devolvido && registos[i].dataDevolucao < dataAtual) {
            // Encontra o livro correspondente no array de livros
            for (int j = 0; j < numLivros; j++) {
                if (strcmp(registos[i].tituloLivro, biblioteca[j].titulo) == 0) {
                    // Imprime o nome do livro, o nome do sócio e os dias em atraso
                    printf("Livro: %s, Sócio: %s, Dias em atraso: %d\n", biblioteca[j].titulo, registos[i].utilizador, dataAtual - registos[i].dataDevolucao);
                    emprestimosEncontrados = 1; // Indica que pelo menos um empréstimo foi encontrado
                    break; // Sai do loop de livros, pois o livro correspondente foi encontrado
                }
            }
        }
    }

    // Se nenhum empréstimo foi encontrado, exibe uma mensagem indicando isso
    if (!emprestimosEncontrados) {
        printf("Nenhum empréstimo em atraso encontrado.\n");
    }
}

void relatorioUtilizadoresMaisEmprestimos(Emprestimo *registos, int numRegistos) {
    printf("Relatório dos 10 utilizadores com mais empréstimos:\n");

    // Inicializa um array para contar o número de empréstimos de cada utilizador
    int contador[numRegistos];
    memset(contador, 0, sizeof(contador)); // Inicializa com 0

    // Conta o número de empréstimos de cada utilizador
    for (int i = 0; i < numRegistos; i++) {
        contador[i] = 1; // Cada empréstimo conta como 1
        for (int j = i + 1; j < numRegistos; j++) {
            if (strcmp(registos[i].utilizador, registos[j].utilizador) == 0) {
                contador[i]++;
            }
        }
    }

    // Encontra e imprime os 10 utilizadores com mais empréstimos
    for (int k = 0; k < 10; k++) {
        int maxEmprestimos = -1;
        int maxIndex = -1;
        for (int i = 0; i < numRegistos; i++) {
            if (contador[i] > maxEmprestimos) {
                maxEmprestimos = contador[i];
                maxIndex = i;
            }
        }

        if (maxIndex != -1) {
            printf("Sócio: %s - %d empréstimos\n", registos[maxIndex].utilizador, maxEmprestimos);
            contador[maxIndex] = -1; // Marcador para evitar que o mesmo usuário seja contado novamente
        }
    }
}